@@ -23,6 +23,7 @@ from ming import schema as S
 import bson
 from bson import ObjectId
 
+import re
 from widgetselector import MingWidgetSelector
 from validatorselector import MingValidatorSelector
 from pymongo import ASCENDING, DESCENDING
@@ -165,6 +166,11 @@ class MingProvider(IProvider):
         """Determine if a field is related to a field in another entity."""
         return isinstance(self.get_field(entity, field_name), RelationProperty)
 
+    def is_query(self, entity, value):
+        """determines if a field is a query instead of actual list of data"""
+        #Currently not supported in MING
+        return False
+
     def is_nullable(self, entity, field_name):
         """Determine if a field is nullable."""
         fld = self.get_field(entity, field_name)
@@ -174,6 +180,11 @@ class MingProvider(IProvider):
         return not getattr(fld, 'kwargs', {}).get("required", False)
 
     def get_field_default(self, field):
+        field = getattr(field, 'field', None)
+        if field is not None:
+            if_missing = field.schema.if_missing
+            if if_missing is not None:
+                return (True, if_missing)
         return (False, None)
 
     def get_field_provider_specific_widget_args(self, entity, field, field_name):
@@ -284,7 +295,14 @@ class MingProvider(IProvider):
         obj.delete()
         return obj
 
-    def query(self, entity, limit=None, offset=0, limit_fields=None, order_by=None, desc=False, filters={}, **kw):
+    def query(self, entity, limit=None, offset=0, limit_fields=None,
+              order_by=None, desc=False, filters={},
+              substring_filters=[], **kw):
+
+        for field in substring_filters:
+            if self.is_string(entity, field):
+                filters[field] = {'$regex':re.compile(re.escape(filters[field]), re.IGNORECASE)}
+
         iter = entity.query.find(filters)
         if offset:
             iter = iter.skip(int(offset))
@@ -299,9 +317,23 @@ class MingProvider(IProvider):
         count = iter.count()
         return count, iter.all()
 
-    def is_binary(self, entity, name):
-        field = self.get_field(entity, name)
-        return isinstance(field,S.Binary)
+    def is_string(self, entity, field_name):
+        fld = self.get_field(entity, field_name)
+        if isinstance(fld, RelationProperty):
+            # check the required attribute on the corresponding foreign key field
+            fld = fld.join.prop
+
+        fld = getattr(fld, 'field', None)
+        return isinstance(fld.schema, S.String)
+
+    def is_binary(self, entity, field_name):
+        fld = self.get_field(entity, field_name)
+        if isinstance(fld, RelationProperty):
+            # check the required attribute on the corresponding foreign key field
+            fld = fld.join.prop
+
+        fld = getattr(fld, 'field', None)
+        return isinstance(fld.schema,S.Binary)
 
     def relation_fields(self, entity, field_name):
         field = self.get_field(entity, field_name)
